import logging
import traceback
from .client import send_sync
from .payload import base_payload

class AIOpsLogHandler(logging.Handler):
    def emit(self, record):
        if record.levelno < logging.ERROR:
            return

        payload = base_payload()
        payload.update({
            "error_type": record.levelname,
            "message": record.getMessage(),
            "stack_trace": record.exc_text or "",
            "source": "logging"
        })

        send_sync("/v1/sdk/exception", payload)

def attach_logging_handler():
    root = logging.getLogger()
    root.addHandler(AIOpsLogHandler())



from .config import load_config
from .exceptions import register_exception_hook, capture_exception
from .scheduler import start_heartbeat
from .logging_hook import attach_logging_handler
from .version import __version__

def init(api_key=None):
    load_config(api_key)
    register_exception_hook()
    attach_logging_handler()
    start_heartbeat()

def init_flask(app):
    from .integrations.flask import init_flask as _init
    _init(app)



import threading
import requests
from .config import Config

def _post(path, payload):
    requests.post(
        f"{Config.platform_url}{path}",
        json=payload,
        headers={
            "Authorization": f"Bearer {Config.api_key}",
            "Content-Type": "application/json"
        },
        timeout=3
    )

def send_async(path, payload):
    threading.Thread(
        target=_safe_post,
        args=(path, payload),
        daemon=True
    ).start()

def send_sync(path, payload):
    try:
        _post(path, payload)
    except Exception:
        pass

def _safe_post(path, payload):
    try:
        _post(path, payload)
    except Exception:
        pass



import os

class Config:
    api_key = None
    service_name = None
    environment = None
    platform_url = None

def load_config(api_key=None):
    Config.api_key = api_key or os.getenv("AIOPS_API_KEY")
    Config.service_name = os.getenv("AIOPS_SERVICE_NAME", "unknown-service")
    Config.environment = os.getenv("AIOPS_ENV", "production")
    Config.platform_url = os.getenv(
        "AIOPS_PLATFORM_URL",
        "https://aiops.1genimpact.cloud"
    )

    if not Config.api_key:
        raise RuntimeError("AIOPS API key is required")



import sys
import traceback
from .client import send_sync
from .payload import base_payload

def _excepthook(exc_type, exc, tb):
    payload = base_payload()
    payload.update({
        "error_type": exc_type.__name__,
        "message": str(exc),
        "stack_trace": "".join(traceback.format_exception(exc_type, exc, tb)),
        "source": "process"
    })
    send_sync("/v1/sdk/exception", payload)

def register_exception_hook():
    sys.excepthook = _excepthook

def capture_exception(exc):
    payload = base_payload()
    payload.update({
        "error_type": type(exc).__name__,
        "message": str(exc),
        "stack_trace": traceback.format_exc(),
        "source": "manual"
    })
    send_sync("/v1/sdk/exception", payload)



from flask import got_request_exception, request
from ..exceptions import capture_exception
from ..payload import base_payload
from ..client import send_sync


def init_flask(app):
    """
    Flask integration for AIOps SDK.

    Captures:
    1. Unhandled Flask exceptions (got_request_exception)
    2. Handled HTTP error responses (4xx / 5xx)
       even when exceptions are swallowed by user code
    """

    # --------------------------------------------------
    # 1️⃣ Capture REAL Flask exceptions (raised errors)
    # --------------------------------------------------
    @got_request_exception.connect_via(app)
    def _handle_exception(sender, exception, **extra):
        capture_exception(exception)

    # --------------------------------------------------
    # 2️⃣ Capture HTTP error responses (status-based)
    # --------------------------------------------------
    @app.after_request
    def _after_request(response):
        try:
            status_code = response.status_code

            # Capture ALL relevant error responses
            if status_code in (401, 403, 404, 500, 503):
                payload = base_payload()
                payload.update({
                    "error_type": "HTTPError",
                    "message": (
                        f"{request.method} {request.path} "
                        f"returned HTTP {status_code}"
                    ),
                    "stack_trace": "",
                    "source": "http",
                    "status_code": status_code
                })

                # Send synchronously but safely
                send_sync("/v1/sdk/exception", payload)

        except Exception:
            # NEVER break the client application
            pass

        return response



from .config import Config
from .utils import hostname, timestamp

def base_payload():
    return {
        "service": Config.service_name,
        "environment": Config.environment,
        "host": hostname(),
        "timestamp": timestamp()
    }



import threading
import time
from .heartbeat import send_heartbeat

def start_heartbeat(interval=30):
    def _run():
        while True:
            send_heartbeat()
            time.sleep(interval)

    thread = threading.Thread(target=_run, daemon=True)
    thread.start()
